/*****************************************************************************
 *                                                                           *
SC 29 Software Copyright Licencing Disclaimer:

This software module was originally developed by
  Mitsuyuki Hatanaka (Sony Corporation),
  Toru Chinen        (Sony Corporation)

and edited by
  -

in the course of development of the ISO/IEC 13818-7 and ISO/IEC 14496-3 
standards for reference purposes and its performance may not have been 
optimized. This software module is an implementation of one or more tools as 
specified by the ISO/IEC 13818-7 and ISO/IEC 14496-3 standards.
ISO/IEC gives users free license to this software module or modifications 
thereof for use in products claiming conformance to audiovisual and 
image-coding related ITU Recommendations and/or ISO/IEC International 
Standards. ISO/IEC gives users the same free license to this software module or 
modifications thereof for research purposes and further ISO/IEC standardisation.
Those intending to use this software module in products are advised that its 
use may infringe existing patents. ISO/IEC have no liability for use of this 
software module or modifications thereof. Copyright is not released for 
products that do not conform to audiovisual and image-coding related ITU 
Recommendations and/or ISO/IEC International Standards.
The original developer retains full right to modify and use the code for its 
own purpose, assign or donate the code to a third party and to inhibit third 
parties from using the code for products that do not conform to audiovisual and 
image-coding related ITU Recommendations and/or ISO/IEC International Standards.
This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 1996.
 *                                                                           *
 ****************************************************************************/
#ifdef AMD4_S

#include <math.h>
#include <stdio.h>
#include "config.h"
#include "downmix.h"

const double S_mixdown_idx[8]={1.0f,0.841f,0.707f,0.596,0.5,0.422f,0.355f,0.0f};
const double LFE_mixdown_idx[16]={3.162f,2.00f,1.679f,1.413f, 1.189f, 1.0f, 0.841f,0.707f, 0.596f, 0.5f, 0.316f, 0.178f, 0.100f, 0.032f, 0.01f, 0.00f};

/*******************************************************************************
 Function name:matrix_mixdown51to2ch
 *******************************************************************************

 Description: 

   This Function is used for downmixing 5.1 channel configuration to 2ch when matrix_mix.present = 1 
   in PCE    
   Based on matrix_mix.pseudo_enab flag in PCE either Set1 or Set2 used for downmix

   SET1:

   a  = [ 1.0/ [1.0+1.0/sqrt(2.0) + A ] ]
   b  = 1.0/sqrt(2.0)

   L' = a * [ L + C*b  + A * Ls ];		
   R' = a * [ R + C*b  + A * Rs ];


   SET2:

   a  = [ 1.0/ [1.0+1.0/sqrt(2.0) + 2*A ] ]
   b  = 1.0/sqrt(2.0)

   L' = a * [ L + C*b  - A * [Ls + Rs ] ];		
   R' = a * [ R + C*b  + A * [Ls + Rs ] ];

   Based on matrix_mix.ele_tag value in PCE, value of A is derived


 Arguments: 
   double **output_data          -- input/output ,pointers to the  data
   int block_length              -- input parameter for length of the block
   int stereo_downmix_mode_LtRt  -- input parameter for stereo downmix mode
			
 Return:    none

*******************************************************************************/
void 
matrix_mixdown51to2ch(double **output_data, int block_length, int stereo_downmix_mode_LtRt)
{
  /* MPEG's downmix 5.1ch to 2ch */
  float A;
  float C,L,R,Ls,Rs;
  int sample;
  float a,b;
 
  switch(prog_config.matrix_mix.ele_tag){
  case 0:
    A = 1.0/sqrt(2.0);
    break;
  case 1:
    A = 0.5f;
    break;
  case 2:
    A = 1.0/(2*sqrt(2.0));
    break;
  case 3:
    A = 0.0f;
    break;
  default:
    A = 1.0f;
    break;
  }

  if(prog_config.matrix_mix.pseudo_enab == 0 || !stereo_downmix_mode_LtRt){
    /* Set 1 equations for downmix */
    a = 1.0/(1.0+1.0/sqrt(2.0)+A);
    b = 1.0/sqrt(2.0);
    for(sample = 0; sample < block_length; sample++)
      {
        C   = output_data[0][sample]; /* front Center */
        L   = output_data[1][sample]; /* front Left */
        R   = output_data[2][sample]; /* front Right */   
        Ls  = output_data[33][sample]; /* back Left */
        Rs  = output_data[34][sample];  /* back Right */

        output_data[1][sample] = a*(L + b*C + A*Ls); /* front Left */
        output_data[2][sample] = a*(R + b*C + A*Rs); /* front Right */

      }
  }else if(prog_config.matrix_mix.pseudo_enab == 1)
    {
      /* Set 2 equations for downmix */
      a = 1.0/(1.0+1.0/sqrt(2.0)+2.0f*A);
      b = 1.0/sqrt(2.0);
      for(sample = 0; sample < block_length; sample++)
        {
          C   = output_data[0][sample]; /* front Center */
          L   = output_data[1][sample]; /* front Left */
          R   = output_data[2][sample]; /* front Right */   
          Ls  = output_data[33][sample]; /* back Left */
          Rs  = output_data[34][sample];  /* back Right */

          output_data[1][sample] = a*(L + b*C - A*(Ls+Rs)); /* front Left */
          output_data[2][sample] = a*(R + b*C + A*(Ls+Rs)); /* front Right */   

        }
    }

}

/*******************************************************************************
 Function name:downmix51to2ch
 *******************************************************************************

 Description: 

   This Function is used for downmixing 5.1/5.0 channel configuration to 2ch when downmixing_levels_MPEG4_status = 1  in DSE    
   Based on pseudo_surround_enable flag in DSE either SET1 or SET2 used for downmix

   SET1:
   L' =  A*[L + C*b + Ls*a + LFE*c]
   R' =  A*[R + C*b + Rs*a + LFE*c]

   Without LFE:
   L' =  A*[L + C*b + Ls*a]
   R' =  A*[R + C*b + Rs*a]

   SET2:
   L' = A*[L + C*b - (Ls+Rs)*a + LFE*c]
   R' = A*[R + C*b + (Ls+Rs)*a + LFE*c] 

   Without LFE:
   L' = A*[L + C*b - (Ls+Rs)*a]
   R' = A*[R + C*b + (Ls+Rs)*a] 

   A is downmix global gain.
   a is surround mix level
   b is center mix level
   c is lfe mix level


 Arguments: 
   double **output_data  -- input/output ,pointers to the  data
   int block_length   -- input parameter for length of the block
   int stereo_downmix_mode_LtRt -- input parameter for stereo downmix mode
   

 Return:    none

*******************************************************************************/
void 
downmix51to2ch(double **output_data, int block_length, int stereo_downmix_mode_LtRt, int hasLFE)
{
  /* MPEG's downmix 5.1ch to 2ch */
  float A;
  float C,L,R,Ls,Rs,LFE;
  int sample;
  float a,b,c;
 

  c = LFE_mixdown_idx[data_config.dmix_lfe_idx];
  
  a = S_mixdown_idx[data_config.surround_mix_level_value];
  b = S_mixdown_idx[data_config.center_mix_level_value];

  if (data_config.dmx_gain_2_sign == 0){
    A = pow(10,data_config.dmx_gain_2_idx / 80.0);
  }else{
    A = pow(10,-data_config.dmx_gain_2_idx / 80.0);
  }

  if(data_config.pseudo_surround_enable == 0 || !stereo_downmix_mode_LtRt)
  {  
    for(sample = 0; sample < block_length; sample++)
    {
      C   = output_data[0][sample]; /* front Center */
      L   = output_data[1][sample]; /* front Left */
      R   = output_data[2][sample]; /* front Right */   
      Ls  = output_data[33][sample]; /* back Left */
      Rs  = output_data[34][sample];  /* back Right */
      if (hasLFE)
      {
        LFE = output_data[48][sample];  /* LFE*/

        output_data[1][sample] = A*(L + C*b + Ls*a + LFE*c); /* Front Left */
        output_data[2][sample] = A*(R + C*b + Rs*a + LFE*c); /* Front Right */
      } else
      {
        output_data[1][sample] = A*(L + C*b + Ls*a); /* Front Left */
        output_data[2][sample] = A*(R + C*b + Rs*a); /* Front Right */
      }
    }

  }else
  {
    for(sample = 0; sample < block_length; sample++)
    {   
      C   = output_data[0][sample]; /* front Center */
      L   = output_data[1][sample]; /* front Left */
      R   = output_data[2][sample]; /* front Right */   
      Ls  = output_data[33][sample]; /* back Left */
      Rs  = output_data[34][sample];  /* back Right */
      if (hasLFE)
      {
        LFE = output_data[48][sample];  /* LFE*/

        output_data[1][sample] = A*(L + C*b - (Ls + Rs)*a + LFE*c); /* Front Left  */
        output_data[2][sample] = A*(R + C*b + (Ls + Rs)*a + LFE*c); /* Front Right */
      } else
      {
        output_data[1][sample] = A*(L + C*b - (Ls + Rs)*a + LFE*c); /* Front Left  */
        output_data[2][sample] = A*(R + C*b + (Ls + Rs)*a + LFE*c); /* Front Right */
      }
    }  
  }
}

/*******************************************************************************
 Function name:downmix61to51ch
 *******************************************************************************

 Description: 

   This Function is used for downmixing 6.1 channel configuration to 5.1ch when ext_downmixing_levels_status == 1 
   in DSE    

   C'   = A*C
   L'   = A*L
   R'   = A*R
   L'   = A*[Ls*a + Cs*b]
   Rs'  = A*[Rs*a + Cs*b]
   LFE' = A*LFE


   A is downmix global gain.
   a is surround mix level
   b is center back mix level  


 Arguments: 
   double **output_data  -- input/output ,pointers to the  data
   int block_length   -- input parameter for length of the block
   

 Return:    none

*******************************************************************************/
void downmix61to51ch(double **output_data, int block_length)
{
  float A;
  float C,L,R,Ls,Rs,Cs,LFE;
  int sample;
  float a,b; 

  a = S_mixdown_idx[data_config.dmix_a_idx];
  b = S_mixdown_idx[data_config.dmix_b_idx];

  if (data_config.dmx_gain_5_sign == 0){
    A = pow(10,data_config.dmx_gain_5_idx / 80.0);
  }else{
    A = pow(10,-data_config.dmx_gain_5_idx / 80.0);
  }

  for(sample = 0; sample < block_length; sample++)
  {
    C   = output_data[0][sample]; /* front Center */
    L   = output_data[1][sample]; /* front Left */
    R   = output_data[2][sample]; /* front Right */   
    Ls  = output_data[33][sample]; /* back Left */
    Rs  = output_data[34][sample];  /* back Right */
    Cs  = output_data[35][sample];  /* back Center */
    LFE = output_data[48][sample];  /* LFE*/

    output_data[0][sample]  = A*C;    /* Front Center */
    output_data[1][sample]  = A*L;    /* Front Left  */
    output_data[2][sample]  = A*R;    /* Front Right */
    output_data[33][sample] = A*(Ls*a  + Cs*b); /* Back Left */
    output_data[34][sample] = A*(Rs*a  + Cs*b); /* Back Right */
    output_data[48][sample] = A*LFE;     /* LFE */
  }
}

/*******************************************************************************
 Function name:downmix71Frontto51ch
 *******************************************************************************

 Description: 

   This Function is used for downmixing 7.1 Front  channel configuration to 5.1 ch when ext_downmixing_levels_status == 1 
   in DSE    

 C'   = A*[C + (Lc+Rc) *a]
 L'   = A*[L + Lc*b]
 R'   = A*[R + Rc*b]
 Ls'  = A*Ls
 Rs'  = A*Rs
 LFE' = A*LFE


   A is downmix global gain.
   a is Center mix level
   b is Front mix level  


 Arguments: 
   double **output_data  -- input/output ,pointers to the  data
   int block_length   -- input parameter for length of the block
   

 Return:    none

*******************************************************************************/
void downmix71Frontto51ch(double **output_data, int block_length)
{
  float A;
  float C,Lc,Rc,L,R,Ls,Rs,LFE;
  int sample;
  float a,b;
 

  a = S_mixdown_idx[data_config.dmix_a_idx];
  b = S_mixdown_idx[data_config.dmix_b_idx];

  if (data_config.dmx_gain_5_sign == 0){
    A = pow(10,data_config.dmx_gain_5_idx / 80.0);
  }else{
    A = pow(10,-data_config.dmx_gain_5_idx / 80.0);
  }

  for(sample = 0; sample < block_length; sample++)
  {
    C   = output_data[0][sample];   /* front Center */
    Lc  = output_data[1][sample];   /* Outside Front Left  */
    Rc  = output_data[2][sample];   /* Outside Front Right  */
    L   = output_data[3][sample];   /* front Left */
    R   = output_data[4][sample];   /* front Right */  
    Ls  = output_data[33][sample];  /* back Left */
    Rs  = output_data[34][sample];  /* back Right */  
    LFE = output_data[48][sample];  /* LFE*/

    output_data[0][sample]  = A*(C + (Lc+Rc)*a); /* Front Center */
    output_data[1][sample]  = A*(L + Lc*b);      /* Front Left  */
    output_data[2][sample]  = A*(R + Rc*b);      /* Front Right */
    output_data[33][sample] = A*Ls;       /* Back Left */
    output_data[34][sample] = A*Rs;       /* Back Right */
    output_data[48][sample] = A*LFE;      /* LFE  */
  }
}

/*******************************************************************************
 Function name:downmix71SurBackto51ch
 *******************************************************************************

 Description: 

   This Function is used for downmixing 7.1 SurBack  channel configuration to 5.1 ch when ext_downmixing_levels_status == 1 
   in DSE

   C'   = A*C
   L'   = A*L
   R'   = A*R
   Ls'  = A*[Ls*a + Lrs*b]
   Rs'  = A*[Rs*a + RrsEb]
   LFE' = A*LFE

   A is downmix global gain.
   a is Surround  mix level
   b is Rear mix level 


 Arguments: 
   double **output_data  -- input/output ,pointers to the  data
   int block_length   -- input parameter for length of the block


 Return:    none

*******************************************************************************/

void downmix71SurBackto51ch(double **output_data, int block_length)
{
  float A;
  float C,L,R,Ls,Rs,Lrs,Rrs,LFE;
  int sample;
  float a,b; 

  a = S_mixdown_idx[data_config.dmix_a_idx];
  b = S_mixdown_idx[data_config.dmix_b_idx];

  if (data_config.dmx_gain_5_sign == 0){
    A = pow(10,data_config.dmx_gain_5_idx / 80.0);
  }else{
    A = pow(10,-data_config.dmx_gain_5_idx / 80.0);
  }

  for(sample = 0; sample < block_length; sample++)
  {
    C   = output_data[0][sample]; /* front Center */
    L   = output_data[1][sample]; /* front Left */
    R   = output_data[2][sample]; /* front Right */  
    Ls  = output_data[33][sample]; /* Surround Left */
    Rs  = output_data[34][sample]; /* Surround Right */
    Lrs = output_data[35][sample];  /* Rear Surround Left  */
    Rrs = output_data[36][sample];  /* Rear Surround Right  */
    LFE = output_data[48][sample];  /* LFE*/

    output_data[0][sample]  = A*C;    /* Front Center */
    output_data[1][sample]  = A*L;             /* Front Left  */
    output_data[2][sample]  = A*R;             /* Front Right */
    output_data[33][sample] = A*(Ls*a + Lrs*b); /* Back Left */
    output_data[34][sample] = A*(Rs*a + Rrs*b); /* Back Right */
    output_data[48][sample] = A*LFE;  /* LFE */
  }
}

/*******************************************************************************
 Function name:downmix71FHto51ch
 *******************************************************************************

 Description: 

   This Function is used for downmixing 7.1 Front Hight channel configuration to 5.1 ch when ext_downmixing_levels_status == 1 
   in DSE    

   C'   = A*C
   L'   = A*[L*a + Lvh*b]
   R'   = A*[R*a + Rvh*b]
   Ls'  = A*Ls
   Rs'  = A*Rs
   LFE' = A*LFE



   A is downmix global gain.
   a is Front mix level
   b is Front Hight mix level  


 Arguments: 
   double **output_data  -- input/output ,pointers to the  data
   int block_length   -- input parameter for length of the block
   

 Return:    none

*******************************************************************************/
void downmix71FHto51ch(double **output_data, int block_length)
{
  float A;
  float C,L,R,Lvh,Rvh,Ls,Rs,LFE;
  int sample;
  float a,b;
 

  a = S_mixdown_idx[data_config.dmix_a_idx];
  b = S_mixdown_idx[data_config.dmix_b_idx];

  if (data_config.dmx_gain_5_sign == 0){
    A = pow(10,data_config.dmx_gain_5_idx / 80.0);
  }else{
    A = pow(10,-data_config.dmx_gain_5_idx / 80.0);
  }

  for(sample = 0; sample < block_length; sample++)
  {
    C   = output_data[0][sample]; /* front Center */
    L   = output_data[1][sample]; /* front Left */
    R   = output_data[2][sample]; /* front Right */
    Lvh = output_data[3][sample];   /* Front Hight Left  */
    Rvh = output_data[4][sample];   /* Front Hight Right  */  
    Ls  = output_data[33][sample]; /* back Left */
    Rs  = output_data[34][sample];  /* back Right */  
    LFE = output_data[48][sample];  /* LFE*/

    output_data[0][sample]  = A*C;       /* Front Center */
    output_data[1][sample]  = A*(L*a + Lvh*b); /* Front Left  */
    output_data[2][sample]  = A*(R*a + Rvh*b); /* Front Right */
    output_data[33][sample] = A*Ls;          /* Back Left */
    output_data[34][sample] = A*Rs;          /* Back Right */
    output_data[48][sample] = A*LFE;         /* LFE */
  }
}

/*******************************************************************************
 Function name:downmix
 *******************************************************************************

 Description: 

   This Function is a generic downmix function,based on the type of the downmix selected/to be performed  it calls one of the downmix function.
   
 Arguments: 
   double **output_data  -- input/output ,pointers to the  data
   int block_length   -- input parameter for length of the block
   
 Return:    none

*******************************************************************************/

void 
downmix(double **output_data, int block_length)
{
  int dmixtype = mc_info.dmixtype;
  MC_Info *p_mcinfo;
  int channel_configuration;

  p_mcinfo = &mc_info; 

  if (p_mcinfo->nch     == 7
      &&p_mcinfo->nfsce == 1
      &&p_mcinfo->nfch  == 3 
      &&p_mcinfo->nsch  == 0 
      && p_mcinfo->nbch == 3 
      && p_mcinfo->nlch == 1     
      ) 
  {
    channel_configuration = 1; /* 6.1 Ch */
  }
  else if (p_mcinfo->nch == 8
           &&p_mcinfo->nfsce  == 1
           &&p_mcinfo->nfch   == 5
           &&p_mcinfo->nsch   == 0 
           && p_mcinfo->nbch  == 2 
           && p_mcinfo->nlch  == 1
           && p_mcinfo->front_element_height_info[2] == 0
           ) 
  {
    channel_configuration = 0; /* 7.1 Front */
  }
  else if (p_mcinfo->nch == 8
           &&p_mcinfo->nfsce  == 1
           &&p_mcinfo->nfch   == 5
           &&p_mcinfo->nsch   == 0 
           && p_mcinfo->nbch  == 2 
           && p_mcinfo->nlch  == 1
           && p_mcinfo->front_element_height_info[2] == 1
           ) 
  {
    channel_configuration = 2; /* 7.1 Front Height */
  }
  else if (p_mcinfo->nch == 8
           &&p_mcinfo->nfsce  == 1
           &&p_mcinfo->nfch   == 3
           &&p_mcinfo->nsch   == 0 
           && p_mcinfo->nbch  == 4 
           && p_mcinfo->nlch  == 1
           ) 
  {
    channel_configuration = 3; /* 7.1 SurBack */
  }
  else
  {
    channel_configuration = -1;
  } 


  p_mcinfo->dmixedout = 0;

  /* Down Mixing to 5.1 Channel */
  if(((dmixtype == 2)||(dmixtype == 3)||(dmixtype == 4)||(dmixtype == 5))
    && (p_mcinfo->nch == 8 || p_mcinfo->nch == 7))
  {  
    if(channel_configuration == 1)
    {
      /* 6.1ch to 5.1ch downmix */
      downmix61to51ch(output_data,block_length);
      p_mcinfo->dmixedout = 51;  /* downmixed output is 5.1 channel */

    }
    else if(channel_configuration == 2)
    {
      /* 7.1ch Front Hight  to 5.1ch downmix */
      downmix71FHto51ch(output_data,block_length);
      p_mcinfo->dmixedout = 51;  /* downmixed output is 5.1 channel */

    }else if(channel_configuration == 3)
    {
      /* 7.1 SurBack to 5.1ch downmix */
      downmix71SurBackto51ch(output_data,block_length);
      p_mcinfo->dmixedout = 51;  /* downmixed output is 5.1 channel */

    }else if(channel_configuration == 0)
    {
      /* 7.1ch Front to 5.1ch downmix */
      downmix71Frontto51ch(output_data,block_length);
      p_mcinfo->dmixedout = 51; /* downmixed output is 5.1 channel */
    }
  }

  /* 5.1ch to 2.1ch */
  if(((dmixtype == 3 || dmixtype == 5))
    && (p_mcinfo->nch == 6 || p_mcinfo->nch == 7 || p_mcinfo->nch == 8))
  {
    /* Function call  for 51 to 2ch mixdown using DSE  */
    downmix51to2ch(output_data,block_length,(p_mcinfo->stereo_downmix_mode_LtRt && dmixtype == 3),p_mcinfo->nlch==1);
    p_mcinfo->dmixedout = 20;  /* downmixed output is 2.0 channel */
    if (dmixtype == 5)
    {
      /* Function call for 2ch to 1ch mixdown */
      downmix2chto1ch(output_data,block_length);
      p_mcinfo->dmixedout = 10;  /* downmixed output is 1.0 channel */
    }
  }else if(dmixtype == 1 && p_mcinfo->nch == 6)
  {
    /* Function call  for 51 to 2ch matrix mixdown */
       matrix_mixdown51to2ch(output_data,block_length,p_mcinfo->stereo_downmix_mode_LtRt);
       p_mcinfo->dmixedout = 20;  /* downmixed output is 2.0 channel */
  }else if(dmixtype == 4 && p_mcinfo->nch == 6)
  {
    /* Function call  for 51 to 1ch matrix mixdown */
       matrix_mixdown51to1ch(output_data,block_length);
       p_mcinfo->dmixedout = 10;  /* downmixed output is 1.0 channel */
  }

  /* 2ch to 1ch */
  if(dmixtype == 5 && p_mcinfo->nch == 2)
  {
    /* Function call for 2ch to 1ch mixdown */
    downmix2chto1ch(output_data,block_length);
    p_mcinfo->dmixedout = 10;  /* downmixed output is 1.0 channel */
  }
}

/*******************************************************************************
 Function name:downmix2chto1ch
 *******************************************************************************

 Description: 

   This Function is used for downmixing stereo channel configuration to mono when downmixing_levels_MPEG4_status = 1  in DSE    
   
   M = L + R
   
 Arguments: 
   double **output_data  -- input/output ,pointers to the  data
   int block_length   -- input parameter for length of the block
   

 Return:    none

*******************************************************************************/
void 
downmix2chto1ch(double **output_data, int block_length)
{
  float L,R;
  int sample;

  for(sample = 0; sample < block_length; sample++)
    {
      L   = output_data[1][sample]; /* Left */
      R   = output_data[2][sample]; /* Right */   
      
      output_data[0][sample] = L + R; /* Center */

    }
}

/*******************************************************************************
 Function name:matrix_mixdown51to1ch
 *******************************************************************************

 Description: 

   This Function is used for downmixing 5.1 channel configuration to 1ch when matrix_mix.present = 1 
   in PCE    
   
   a  = [ 1.0/ [3.0 + 2.0*A ] ]
   
   M = a * [ C + L + R + A * [Ls + Rs ] ];		
   
   Based on matrix_mix.ele_tag value in PCE, value of A is derived

 Arguments: 
			double **output_data		-- input/output ,pointers to the  data
			int block_length			-- input parameter for length of the block
			
 Return:    none

*******************************************************************************/
void 
matrix_mixdown51to1ch(double **output_data, int block_length)
{
  /* MPEG's downmix 5.1ch to 1ch */
  float A;
  float C,L,R,Ls,Rs;
  int sample;
  float a;
 
  switch(prog_config.matrix_mix.ele_tag){
  case 0:
    A = 1.0/sqrt(2.0);
    break;
  case 1:
    A = 0.5f;
    break;
  case 2:
    A = 1.0/(2*sqrt(2.0));
    break;
  case 3:
    A = 0.0f;
    break;
  default:
    A = 1.0f;
    break;
  }

  if(prog_config.matrix_mix.pseudo_enab == 0){
    /* Set 1 equations for downmix */
    a = 1.0/(3.0+2.0*A);
    for(sample = 0; sample < block_length; sample++)
      {
        C   = output_data[0][sample]; /* front Center */
        L   = output_data[1][sample]; /* front Left */
        R   = output_data[2][sample]; /* front Right */   
        Ls  = output_data[33][sample]; /* back Left */
        Rs  = output_data[34][sample];  /* back Right */

        output_data[0][sample] = a*(C + L + R + A*(Ls+Rs)); /* Center */
        
      }
  }
}

#endif /* AMD4_S */
