/*

This software module was originally developed by

    Shusuke Takahashi (Sony Corporation)

    in the course of development of the MPEG-4 Audio standard (ISO/IEC 14496-3).
    This software module is an implementation of a part of one or more
    MPEG-4 Audio (ISO/IEC 14496-3) tools as specified by the MPEG-4 Audio
    standard (ISO/IEC 14496-3).
    ISO/IEC gives users of the MPEG-4 Audio standards (ISO/IEC 14496-3)
    free license to this software module or modifications thereof for use
    in hardware or software products claiming conformance to the MPEG-4
    Audio standards (ISO/IEC 14496-3).
    Those intending to use this software module in hardware or software
    products are advised that this use may infringe existing patents.
    The original developer of this software module and his/her company,
    the subsequent editors and their companies, and ISO/IEC have no
    liability for use of this software module or modifications thereof in
    an implementation.
    Copyright is not released for non MPEG-4 Audio (ISO/IEC 14496-3)
    conforming products. The original developer retains full right to use
    the code for his/her own purpose, assign or donate the code to a third
    party and to inhibit third party from using the code for non MPEG-4
    Audio (ISO/IEC 14496-3) conforming products.
    This copyright notice must be included in all copies or derivative works.

    Copyright (c)2015.

*/
#include <stdio.h>
#include <stdlib.h>

#include "libmpegasfe.h"
#include "wavefile.h"

#define MPEG_AS_OK 0
#define MPEG_AS_NG 1

#define	AUDIO_SYNC_FEATURE_TYPE_0 0
#define	AUDIO_SYNC_FEATURE_FRAME_LENGTH_INDEX_OF_128 0
#define	AUDIO_SYNC_FEATURE_TIME_RESOLUTION_INDEX_OF_32 0
#define	AUDIO_SYNC_FEATURE_TIME_RESOLUTION_INDEX_OF_8  1

#define	DEFAULT_RESOLUTION	8

static float s_duration = 0;
static int	 s_resolution = 0;
static char  *s_filename1 = NULL;
static char  *s_filename2 = NULL;

static int get_option(int, char **);
static void show_usage();
static int write_audio_specific_config(int resolution, FILE *fo);

int main(int argc, char* argv[])
{

	MPEG_AS_FE_HANDLE	handle = NULL;	
	char *p_filename_in = NULL;
	char *p_filename_out = NULL;
	MPEG_AS_RESULT	result;
	int frame_size;
	int feature_size;
	int sampling_freq;
	int ret = MPEG_AS_OK;
	char *p_audio_in = NULL;
	char *p_feature = NULL;
	FILE *fi = NULL;
	FILE *fo = NULL;
	RIFFWAVE	riffwave;
	int filelen_sample;
	double filelen_sec;
	int num_frame;
	int	idx_frame;

	fprintf(stderr, " -- MPEG Audio Synchronization (version 1.0) --\n");

	if (get_option(argc, argv) != 0) {
		show_usage();
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	p_filename_in = s_filename1;
	if (p_filename_in == NULL) {
		printf("filename error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	p_filename_out = s_filename2;
	if (p_filename_out == NULL) {
		printf("filename error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}
	
	/* get handle */
	handle = mpeg_as_fe_get_handle();
	if (handle == NULL) {
		fprintf(stderr, "error: mpeg_as_fe_get_handle()\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	/* get parameters */
	result = mpeg_as_fe_get_frame_size(handle, &frame_size);
	if (MPEG_AS_FAILED(result)) {
		printf("mpeg_as_fe_get_frame_size error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	result = mpeg_as_fe_get_feature_size(handle, &feature_size);
	if (MPEG_AS_FAILED(result)) {
		printf("mpeg_as_fe_get_feature_size error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	result = mpeg_as_fe_get_sampling_rate(handle, &sampling_freq);
	if (MPEG_AS_FAILED(result)) {
		printf("mpeg_as_fe_get_sampling_rate error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	/* set parameters */
	if (s_resolution == 0) {
		s_resolution = DEFAULT_RESOLUTION;
	} 

	result = mpeg_as_fe_set_time_resolution(handle, s_resolution);
	if (MPEG_AS_FAILED(result)) {
		printf("mpeg_as_fe_set_time_resolution error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	/* initialize */
	result = mpeg_as_fe_init_handle(handle);
	if (MPEG_AS_FAILED(result)) {
		printf("mpeg_as_fe_init_handle error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	/* open files */
#if _MSC_VER >=1500 
	result = fopen_s(&fi, p_filename_in, "rb");
	if (result != 0) {
		fprintf(stderr, "can't open input file (%s)\n", p_filename_in);
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}
#else
	fi = fopen(p_filename_in, "rb");
	if (fi == NULL) {
		fprintf(stderr, "can't open input file (%s)\n", p_filename_in);
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}
#endif


#if _MSC_VER >=1500 
	result = fopen_s(&fo, p_filename_out, "wb");
	if (result != 0) {
		fprintf(stderr, "can't open output file (%s)\n", p_filename_out);
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}
#else
	fo = fopen(p_filename_out, "wb");
	if (fo == NULL) {
		fprintf(stderr, "can't open output file (%s)\n", p_filename_out);
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}
#endif
	if (parse_riff_wave_header(fi, &riffwave) < 0) {
		printf("parse_riff_wave_header Error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	if (riffwave.fmt.nSamplesPerSec != sampling_freq ||
		!(riffwave.fmt.nChannels == 1 ||	riffwave.fmt.nChannels == 2) ||
		riffwave.fmt.nBlockAlign / riffwave.fmt.nChannels != 2 ) 
	{
		printf("Input file format error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}

	p_audio_in = (char *)malloc(frame_size*sizeof(short)*riffwave.fmt.nChannels);
	if (p_audio_in == NULL) {
		printf("malloc error: p_audio_in\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}
	
	p_feature = (char *)malloc(feature_size*sizeof(char));
	if (p_feature == NULL) {
		printf("malloc error: p_feature\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}


	/* processing */
	idx_frame = 0;

	filelen_sample = riffwave.data_chunk_size/riffwave.fmt.nBlockAlign;
	filelen_sec = (double)filelen_sample / riffwave.fmt.nSamplesPerSec;

	if (filelen_sec > s_duration && s_duration != 0) {
		filelen_sample = (int)(s_duration * riffwave.fmt.nSamplesPerSec);
		filelen_sec = (double)filelen_sample / riffwave.fmt.nSamplesPerSec;
	}

	num_frame = filelen_sample / frame_size;

	ret = write_audio_specific_config(s_resolution, fo);
	if (ret == MPEG_AS_NG) {
		printf("write_audio_specific_config error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;
	}
	
	fprintf(stderr, " Input file: \n");
	fprintf(stderr, "    Filename:   %s\n", p_filename_in);
	fprintf(stderr, "    Duration:   %f sec\n", filelen_sec);

	fprintf(stderr, "\n");
	fprintf(stderr, " Output file: \n");
	fprintf(stderr, "    Filename:   %s\n", p_filename_out);
	fprintf(stderr, "    Resolution: %d msec\n", s_resolution);

	for (idx_frame = 0; idx_frame < num_frame; idx_frame++) {
		unsigned int nbyte_read;
		unsigned int nbyte_written;
		int is_valid;

		nbyte_read = fread(p_audio_in, sizeof(char),  frame_size*sizeof(short)*riffwave.fmt.nChannels, fi);
		if (nbyte_read != frame_size*sizeof(short)*riffwave.fmt.nChannels) {
			printf("fread error\n");
			ret = MPEG_AS_NG;
			goto FINISH_ROUTINE;
		}

		/* stereo -> mono */
		if (riffwave.fmt.nChannels == 2) {
			int idx_sample;
			for (idx_sample = 0; idx_sample < frame_size; idx_sample++) {
				int val1 = ((short *)p_audio_in)[2*idx_sample];
				int val2 = ((short *)p_audio_in)[2*idx_sample+1];
				((short *)p_audio_in)[idx_sample] = (short)((val1 + val2)/2);
			}
		}
		
		result = mpeg_as_fe_processing(handle, p_audio_in, p_feature, &is_valid);
		if (MPEG_AS_FAILED(result)) {
			printf("mpeg_as_fe_processing error\n");
			ret = MPEG_AS_NG;
			goto FINISH_ROUTINE;
		}

		if (is_valid) {
			nbyte_written = fwrite(p_feature, sizeof(char), feature_size, fo);
			if (nbyte_written != (unsigned int)feature_size) {
				printf("fwrite error\n");
				ret = MPEG_AS_NG;
				goto FINISH_ROUTINE;			
			}
		}
	}

FINISH_ROUTINE:
	/* clean-up */

	if (handle != NULL) {
		mpeg_as_fe_free_handle(handle);
	}

	if (p_audio_in != NULL) {
		free(p_audio_in);
	}

	if (p_feature != NULL) {
		free(p_feature);
	}

	if (fi != NULL) {
		fclose(fi);
	}

	if (fo != NULL) {
		fclose(fo);
	}

	return ret;
}

static int get_option(
	int			argc, 
	char		**argv
)
{
    int		i, j;
    char	*p_ptr;

    for (i = 1, j = 0; i < argc; i++) {
		p_ptr = argv[i];
		if (p_ptr[0] == '-' && p_ptr[1] != 0) {
			p_ptr++;
			if (0) {
				;
			} else if (!strcmp(p_ptr, "h")) {
				return 1;	
			} else if (!strcmp(p_ptr, "ds")) {
				s_duration = (float)atof(argv[++i]);
			} else if (!strcmp(p_ptr, "r")) {
				s_resolution = atoi(argv[++i]);
			} else {
				printf("-%s : unknown option\n", p_ptr);
				return 1;
			}
		} else {
			if (j == 0) {
				s_filename1 = p_ptr;
			} else if (j == 1) {
				s_filename2 = p_ptr;
			}  
			j++;
		}
    }

	if (j != 2) {
		return 1;
	}
	return 0;
}

static void show_usage() {

	fprintf(stderr, "\nusage: mpeg_as_fe [options] inputfile outputfile\n");
	fprintf(stderr, "options:\n");
	fprintf(stderr, "   -ds: set duration in sec for inputfile\n");
	fprintf(stderr, "   -r : set audio_sync_feature_time_resolution (8msec or 32msec)\n");
	return;

}

static int write_audio_specific_config(int resolution, FILE *fo) {
	int number_of_streams_index = 1;
	unsigned int nbyte_written;
	unsigned char a_audio_specific_config[4];
	unsigned int time_resolution;
	unsigned int audio_specific_config_size = 4;
	int ret = MPEG_AS_OK;

	a_audio_specific_config[0] = (((0x0000000f & AUDIO_SYNC_FEATURE_TYPE_0) << 4) | 
		((0x0000000f & AUDIO_SYNC_FEATURE_FRAME_LENGTH_INDEX_OF_128)));

	if (resolution == 8) {
		time_resolution = AUDIO_SYNC_FEATURE_TIME_RESOLUTION_INDEX_OF_8;		
	} else if (resolution == 32) {
		time_resolution = AUDIO_SYNC_FEATURE_TIME_RESOLUTION_INDEX_OF_32;		
	} else {
		printf("invalid time resolution\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;			
	}

	a_audio_specific_config[1] = (((0x0000000f & time_resolution) << 4) | 
		((0x0000000f & number_of_streams_index)));

	a_audio_specific_config[2] = 0; /* reserved */
	a_audio_specific_config[3] = 0; /* reserved */

	nbyte_written = fwrite(a_audio_specific_config, sizeof(char), audio_specific_config_size, fo);
	if (nbyte_written != (unsigned int)audio_specific_config_size) {
		printf("fwrite error\n");
		ret = MPEG_AS_NG;
		goto FINISH_ROUTINE;			
	}

FINISH_ROUTINE:
	return ret;
}
/*******************************************************************************
	End of file
*******************************************************************************/
