/*

This software module was originally developed by

    Shusuke Takahashi (Sony Corporation)

    in the course of development of the MPEG-4 Audio standard (ISO/IEC 14496-3).
    This software module is an implementation of a part of one or more
    MPEG-4 Audio (ISO/IEC 14496-3) tools as specified by the MPEG-4 Audio
    standard (ISO/IEC 14496-3).
    ISO/IEC gives users of the MPEG-4 Audio standards (ISO/IEC 14496-3)
    free license to this software module or modifications thereof for use
    in hardware or software products claiming conformance to the MPEG-4
    Audio standards (ISO/IEC 14496-3).
    Those intending to use this software module in hardware or software
    products are advised that this use may infringe existing patents.
    The original developer of this software module and his/her company,
    the subsequent editors and their companies, and ISO/IEC have no
    liability for use of this software module or modifications thereof in
    an implementation.
    Copyright is not released for non MPEG-4 Audio (ISO/IEC 14496-3)
    conforming products. The original developer retains full right to use
    the code for his/her own purpose, assign or donate the code to a third
    party and to inhibit third party from using the code for non MPEG-4
    Audio (ISO/IEC 14496-3) conforming products.
    This copyright notice must be included in all copies or derivative works.

    Copyright (c)2015.

*/

#include "wavefile.h"

#define RIFF_WAVE_HEADER_SIZE 36
#define EXTENSIBLE_SIZE 24

static short
get_16bit_le(unsigned char *p_header)
{

	short val;
	
	val = (0xff00 & (p_header[1] <<  8)) |
		  (0x00ff & (p_header[0]      ));
	
	return val;
}

static int
get_32bit_le(unsigned char *p_header)
{

	int val;
	
	val = (0xff000000 & (p_header[3] << 24)) |
		  (0x00ff0000 & (p_header[2] << 16)) |
		  (0x0000ff00 & (p_header[1] <<  8)) |
		  (0x000000ff & (p_header[0]      ));
	
	return val;
}

int
parse_riff_wave_header(FILE *fi, RIFFWAVE *p_riffwave)
{

	unsigned char a_header[RIFF_WAVE_HEADER_SIZE];
	unsigned char *p_header;
	unsigned char *p_chunk_name;
	int  bytes_read;
	int  chunk_size;
	int  used_bytes;
	
	bytes_read = fread(a_header, sizeof(char), RIFF_WAVE_HEADER_SIZE, fi);

	if (bytes_read != RIFF_WAVE_HEADER_SIZE) {
		fprintf(stderr, "fread error\n");
		return RIFFWAVE_NG;
	}

	p_header = a_header;

	p_chunk_name = p_header; p_header += 4;
	if (strncmp((const char *)p_chunk_name, "RIFF", 4)) {
		fprintf(stderr, "this is not RIFF header\n");
		return RIFFWAVE_NG;
	}

	p_riffwave->riff_chunk_size = get_32bit_le(p_header); p_header += 4;

	p_chunk_name = p_header; p_header += 4;
	if (strncmp((const char *)p_chunk_name, "WAVE", 4)) {
		fprintf(stderr, "this is not WAVE header\n");
		return RIFFWAVE_NG;
	}

	p_chunk_name = p_header; p_header += 4;
	if (strncmp((const char *)p_chunk_name, "fmt ", 4)) {
		fprintf(stderr, "this is not format chunk\n");
		return RIFFWAVE_NG;
	}

	p_riffwave->fmt.chunk_size = get_32bit_le(p_header); p_header += 4;
	p_riffwave->fmt.wFormatTag = get_16bit_le(p_header); p_header += 2;

	if (p_riffwave->fmt.wFormatTag != 1 && p_riffwave->fmt.wFormatTag != (short)0xfffe) {
		fprintf(stderr, "unknown format id %d\n", p_riffwave->fmt.wFormatTag);
		return RIFFWAVE_NG;
	}

	p_riffwave->fmt.nChannels = get_16bit_le(p_header);       p_header += 2;
	p_riffwave->fmt.nSamplesPerSec = get_32bit_le(p_header);  p_header += 4;
	p_riffwave->fmt.nAvgBytesPerSec = get_32bit_le(p_header); p_header += 4;
	p_riffwave->fmt.nBlockAlign = get_16bit_le(p_header);     p_header += 2;
	p_riffwave->fmt.wBitsPerSample = get_16bit_le(p_header);  p_header += 2;

	used_bytes  = 16;

	if (p_riffwave->fmt.wFormatTag == (short)0xfffe) {
		/* extensible */
		unsigned char a_extensible[EXTENSIBLE_SIZE];
		short tmp_short;
		unsigned long   Data1;
		unsigned short  Data2;
		unsigned short  Data3;
		unsigned char   Data4[8];
		GUID sub_format = {0x00000001,
						   0x0000, 0x0010,
						   {0x80, 0x00, 0x00, 0xaa,
							0x00, 0x38, 0x9b, 0x71}};

		bytes_read = fread(a_extensible, sizeof(char), EXTENSIBLE_SIZE, fi);

		if (bytes_read != EXTENSIBLE_SIZE) {
			fprintf(stderr, "fread error\n");
			return RIFFWAVE_NG;
		}

		p_header = a_extensible;

		tmp_short = get_16bit_le(p_header); p_header += 2;
		if (tmp_short != EXTENSIBLE_SIZE -2) {
			fprintf(stderr, "unsupported EXTENSIBLE format\n");
			return RIFFWAVE_NG;
		}
		
		p_riffwave->fmt.ext.Samples.wReserved = get_16bit_le(p_header); p_header += 2;
		p_riffwave->fmt.ext.dwChannelMask = get_32bit_le(p_header); p_header += 4;

		Data1 = get_32bit_le(p_header); p_header += 4;
		Data2 = get_16bit_le(p_header); p_header += 2;
		Data3 = get_16bit_le(p_header); p_header += 2;
		memcpy(Data4, p_header, 8);     p_header += 8;
 
		if (Data1 != sub_format.Data1 ||
			Data2 != sub_format.Data2 ||
			Data3 != sub_format.Data3 ||
			strncmp((const char *)Data4, (char *)sub_format.Data4, 8)) {
			fprintf(stderr, "unsupported GUID\n");
			return RIFFWAVE_NG;
		}

		used_bytes += EXTENSIBLE_SIZE;
	}

	if (p_riffwave->fmt.chunk_size > used_bytes) {
		int i;
		unsigned char dummy;
		for (i = 0; i < p_riffwave->fmt.chunk_size - used_bytes; i++) {
			bytes_read = fread(&dummy, sizeof(char), 1, fi);
			if (bytes_read != 1) {
				fprintf(stderr, "fread error\n");
				return RIFFWAVE_NG;
			}
		}
	}
	
	while (1) {
		bytes_read = fread(a_header, sizeof(char), 8, fi);
		
		if (bytes_read != 8) {
			fprintf(stderr, "fread error\n");
			return RIFFWAVE_NG;
		}
		
		p_header = a_header;
		p_chunk_name = p_header; p_header += 4;

		if (strncmp((const char *)p_chunk_name, "data", 4)) {
			chunk_size = get_32bit_le(p_header); p_header += 4;
			fseek(fi, chunk_size, SEEK_CUR);
			fprintf(stderr, "unknown chunk (%c%c%c%c) ... skipped\n", p_chunk_name[0], p_chunk_name[1], p_chunk_name[2], p_chunk_name[3]);
		} else {
			p_riffwave->data_chunk_size = get_32bit_le(p_header); p_header += 4;
			break;
		}
	}
	
	return RIFFWAVE_OK;
}

static void
put_16bit_le(unsigned char *p_header, short val)
{

	p_header[0] = (unsigned char)((0x00ff & val));
	p_header[1] = (unsigned char)((0xff00 & val) >> 8);

	return;
}

static void
put_32bit_le(unsigned char *p_header, int val)
{
	
	p_header[0] = (unsigned char)((0x000000ff & val));
	p_header[1] = (unsigned char)((0x0000ff00 & val) >> 8);
	p_header[2] = (unsigned char)((0x00ff0000 & val) >> 16);
	p_header[3] = (unsigned char)((0xff000000 & val) >> 24);
	
	return;
}

int
write_riff_wave_header(FILE *fo, RIFFWAVE *p_riffwave)
{
	
	unsigned char a_header[RIFF_WAVE_HEADER_SIZE];
	unsigned char *p_header;
	unsigned char *p_chunk_name;
	int  dummy;
	int  bytes_written;
	
	p_header = a_header;
	p_chunk_name = p_header; p_header += 4;
	
	p_chunk_name[0] = 'R';
	p_chunk_name[1] = 'I';
	p_chunk_name[2] = 'F';
	p_chunk_name[3] = 'F';

	dummy = 0;
	put_32bit_le(p_header, dummy); p_header += 4;

	p_chunk_name = p_header; p_header += 4;
	
	p_chunk_name[0] = 'W';
	p_chunk_name[1] = 'A';
	p_chunk_name[2] = 'V';
	p_chunk_name[3] = 'E';

	p_chunk_name = p_header; p_header += 4;

	p_chunk_name[0] = 'f';
	p_chunk_name[1] = 'm';
	p_chunk_name[2] = 't';
	p_chunk_name[3] = ' ';

	put_32bit_le(p_header, p_riffwave->fmt.chunk_size);      p_header += 4;
	put_16bit_le(p_header, p_riffwave->fmt.wFormatTag);      p_header += 2;
	put_16bit_le(p_header, p_riffwave->fmt.nChannels);       p_header += 2; 
	put_32bit_le(p_header, p_riffwave->fmt.nSamplesPerSec);  p_header += 4;
	put_32bit_le(p_header, p_riffwave->fmt.nAvgBytesPerSec); p_header += 4;
	put_16bit_le(p_header, p_riffwave->fmt.nBlockAlign);     p_header += 2;
	put_16bit_le(p_header, p_riffwave->fmt.wBitsPerSample);  p_header += 2;
	
	bytes_written = fwrite(a_header, sizeof(char), RIFF_WAVE_HEADER_SIZE, fo);

	if (bytes_written != RIFF_WAVE_HEADER_SIZE) {
		fprintf(stderr, "fwrite error\n");
		return RIFFWAVE_NG;
	}

	if (p_riffwave->fmt.wFormatTag == (short)0xfffe) {
		/* extensible */
		unsigned char a_extensible[EXTENSIBLE_SIZE];
		short tmp_short;
		
		GUID sub_format = {0x00000001,
						   0x0000, 0x0010,
						   {0x80, 0x00, 0x00, 0xaa,
							0x00, 0x38, 0x9b, 0x71}};

		p_header = a_extensible;

		tmp_short = 22;
		put_16bit_le(p_header, tmp_short);                              p_header += 2;
		put_16bit_le(p_header, p_riffwave->fmt.ext.Samples.wReserved);  p_header += 2;
		put_32bit_le(p_header, p_riffwave->fmt.ext.dwChannelMask);      p_header += 4;

		put_32bit_le(p_header, sub_format.Data1);                       p_header += 4;
		put_16bit_le(p_header, sub_format.Data2);                       p_header += 2;
		put_16bit_le(p_header, sub_format.Data3);                       p_header += 2;
		memcpy(p_header, sub_format.Data4, 8);                          p_header += 8; 

		bytes_written = fwrite(a_extensible, sizeof(char), EXTENSIBLE_SIZE, fo);

		if (bytes_written != EXTENSIBLE_SIZE) {
			fprintf(stderr, "fwrite error\n");
			return RIFFWAVE_NG;
		}
	}

	p_header = a_header;
	p_chunk_name = p_header; p_header += 4;

	p_chunk_name[0] = 'd';
	p_chunk_name[1] = 'a';
	p_chunk_name[2] = 't';
	p_chunk_name[3] = 'a';

	dummy = 0;
	put_32bit_le(p_header, dummy); p_header += 4;
		
	bytes_written = fwrite(a_header, sizeof(char), 8, fo);

	if (bytes_written != 8) {
		fprintf(stderr, "fwrite error\n");
		return RIFFWAVE_NG;
	}

	return RIFFWAVE_OK;
}

int
update_riff_wave_header(FILE *fo, RIFFWAVE *p_riffwave)
{

	long filesize;
	unsigned char a_buf[4];
	int header_size;

	header_size = RIFF_WAVE_HEADER_SIZE + 8;
	
	if (p_riffwave->fmt.wFormatTag == (short)0xfffe) {
		header_size += EXTENSIBLE_SIZE;
	} 
	
	filesize = ftell(fo);

	put_32bit_le(a_buf, filesize - 8);
	
	if (fseek(fo, 4, SEEK_SET) < 0) {
		return RIFFWAVE_NG;
	}

	if (fwrite(a_buf, sizeof(char), 4, fo) != 4) {
		return RIFFWAVE_NG;
	}
	
	put_32bit_le(a_buf, filesize - header_size);
	
	if (fseek(fo, header_size - 4, SEEK_SET) < 0) {
		return RIFFWAVE_NG;
	}

	if (fwrite(a_buf, sizeof(char), 4, fo) != 4) {
		return RIFFWAVE_NG;
	}
	
	if (fseek(fo, filesize, SEEK_SET) < 0) {
		return RIFFWAVE_NG;
	}

	return RIFFWAVE_OK;
}

/*******************************************************************************
	End of file
*******************************************************************************/
