/*

This software module was originally developed by

    Shusuke Takahashi (Sony Corporation)

    in the course of development of the MPEG-4 Audio standard (ISO/IEC 14496-3).
    This software module is an implementation of a part of one or more
    MPEG-4 Audio (ISO/IEC 14496-3) tools as specified by the MPEG-4 Audio
    standard (ISO/IEC 14496-3).
    ISO/IEC gives users of the MPEG-4 Audio standards (ISO/IEC 14496-3)
    free license to this software module or modifications thereof for use
    in hardware or software products claiming conformance to the MPEG-4
    Audio standards (ISO/IEC 14496-3).
    Those intending to use this software module in hardware or software
    products are advised that this use may infringe existing patents.
    The original developer of this software module and his/her company,
    the subsequent editors and their companies, and ISO/IEC have no
    liability for use of this software module or modifications thereof in
    an implementation.
    Copyright is not released for non MPEG-4 Audio (ISO/IEC 14496-3)+
    conforming products. The original developer retains full right to use
    the code for his/her own purpose, assign or donate the code to a third
    party and to inhibit third party from using the code for non MPEG-4
    Audio (ISO/IEC 14496-3) conforming products.
    This copyright notice must be included in all copies or derivative works.

    Copyright (c)2015.

*/

#include "as_compare.h"

/* prototype */
static void prepare_block_similarity(
MPEG_AS_SYNC_HANDLE handle, 
const unsigned char *p_buf0, 
const unsigned char *p_buf1, 
int len0, 
int len1, 
int **pp_andval, 
int **pp_orval);

static void calc_block_similarity(
MPEG_AS_SYNC_HANDLE handle, 
int **pp_andval, 
int **pp_orval,
int len0, 
int len1
);

static int check_route_length(
MPEG_AS_SYNC_HANDLE handle,
int	len0,
int	len1,
int	*p_max_num_point
);

static void normalize_scoresum(
MPEG_AS_SYNC_HANDLE handle, 
int len0,
int len1,
int max_num_point
);

static void get_max_val_of_scoresum(
MPEG_AS_SYNC_HANDLE handle, 
int len0, 
int len1, 
float *p_maxval, 
int *p_maxval_id
);

/* functions */

/* compare features */
int audio_sync_compare_features
(
MPEG_AS_SYNC_HANDLE handle,
const unsigned char *p_buf0,
const int len0,
const unsigned char *p_buf1,
const int len1
) 
{
	int ret = MPEG_AS_S_OK;

	int **pp_orval = NULL;
	int **pp_andval = NULL;
	float *a_scoresum = NULL;
	int	idx_frame0;


	if (len0 < MPEG_AS_BLOCK_SIZE) {
		fprintf(stderr, "input file size is too small\n");
		return MPEG_AS_E_SHORT_INPUT;
	}

	if (len1 < MPEG_AS_BLOCK_SIZE) {
		fprintf(stderr, "input file size is too small\n");
		return MPEG_AS_E_SHORT_INPUT;
	}

	pp_andval = (int **)mpeg_as_calloc(len0, sizeof(int *));
	if (pp_andval == NULL) {
		fprintf(stderr, "malloc error\n");
		ret = MPEG_AS_E_OUT_OF_MEMORY;
		goto FINISH_ROUTINE;
	}

	pp_orval = (int **)mpeg_as_calloc(len0, sizeof(int *));
	if (pp_orval == NULL) {
		fprintf(stderr, "malloc error\n");
		ret = MPEG_AS_E_OUT_OF_MEMORY;
		goto FINISH_ROUTINE;
	}

	for (idx_frame0 = 0; idx_frame0 < len0; idx_frame0++) {
		pp_andval[idx_frame0] = (int *)mpeg_as_calloc(len1, sizeof(int));
		if (pp_andval[idx_frame0] == NULL) {
			fprintf(stderr, "malloc error\n");
			ret = MPEG_AS_E_OUT_OF_MEMORY;
			goto FINISH_ROUTINE;
		}

		pp_orval[idx_frame0] = (int *)mpeg_as_calloc(len1, sizeof(int));
		if (pp_orval[idx_frame0] == NULL) {
			fprintf(stderr, "malloc error\n");
			ret = MPEG_AS_E_OUT_OF_MEMORY;
			goto FINISH_ROUTINE;
		}
	}

	a_scoresum = (float *)mpeg_as_calloc(len0 + len1 - 1, sizeof(float));
	if (a_scoresum == NULL) {
		fprintf(stderr, "malloc error\n");
		ret = MPEG_AS_E_OUT_OF_MEMORY;
		goto FINISH_ROUTINE;
	}

	prepare_block_similarity(handle, p_buf0, p_buf1, len0, len1, pp_andval, pp_orval);

	calc_block_similarity(handle, pp_andval, pp_orval, len0, len1);


FINISH_ROUTINE:
	for (idx_frame0 = 0; idx_frame0 < len0; idx_frame0++) {
		if (pp_andval[idx_frame0] != NULL) {
			mpeg_as_free(pp_andval[idx_frame0]);
		}
	}

	if (pp_andval != NULL) {
		mpeg_as_free(pp_andval);
	}

	for (idx_frame0 = 0; idx_frame0 < len0; idx_frame0++) {
		if (pp_orval[idx_frame0] != NULL) {
			mpeg_as_free(pp_orval[idx_frame0]);
		}
	}

	if (pp_orval != NULL) {
		mpeg_as_free(pp_orval);
	}

	if (a_scoresum != NULL) {
		mpeg_as_free(a_scoresum);
	}

	return ret;
}

int audio_sync_estimate_time_difference(
MPEG_AS_SYNC_HANDLE handle, 
float *p_maxval,
int	  *p_maxval_id
) {
	int len0 = handle->len_rate_convert0;
	int len1 = handle->len_rate_convert1;

	int max_num_point;

	int ret;

	/* check length */
	ret = check_route_length(handle, len0, len1, &max_num_point);
	if (ret == MPEG_AS_E_SHORT_INPUT) {
		return ret;
	}

	/* normalize */ 
	normalize_scoresum(handle, len0, len1, max_num_point);

	/* get max val */
	get_max_val_of_scoresum(handle, len0, len1, p_maxval, p_maxval_id);

	return MPEG_AS_S_OK;
}

static void prepare_block_similarity(
MPEG_AS_SYNC_HANDLE handle, 
const unsigned char *p_buf0, 
const unsigned char *p_buf1, 
int len0, 
int len1, 
int **pp_andval, 
int **pp_orval
)
{
	int idx_frame0, idx_frame1;
	for (idx_frame0 = 0; idx_frame0 < len0; idx_frame0++) {
		int startpos;
		int endpos;

		startpos = 0;
		endpos = len1;

		if (handle->search_width > 0) {
			startpos = idx_frame0 - (int)(handle->search_width/handle->framestep_new);
			endpos = idx_frame0 + (int)(handle->search_width/handle->framestep_new) + 1;
			if (startpos < 0) {
				startpos = 0;
			}

			if (endpos > len1) {
				endpos = len1;
			}
		}

		for (idx_frame1 = startpos; idx_frame1 < endpos; idx_frame1++) {

			int andval, orval;
			int idx_byte;

			andval = 0;
			orval = 0;

			for (idx_byte = 0; idx_byte < MPEG_AS_NUM_BYTE_FEAT; idx_byte++) {
				andval += g_a_popcnt[p_buf0[idx_frame0*MPEG_AS_NUM_BYTE_FEAT + idx_byte] & p_buf1[idx_frame1*MPEG_AS_NUM_BYTE_FEAT + idx_byte]];
				orval += g_a_popcnt[p_buf0[idx_frame0*MPEG_AS_NUM_BYTE_FEAT + idx_byte] | p_buf1[idx_frame1*MPEG_AS_NUM_BYTE_FEAT + idx_byte]];
			}

			pp_andval[idx_frame0][idx_frame1] = andval;
			pp_orval[idx_frame0][idx_frame1] = orval;
		}
	}

}

static void calc_block_similarity(
MPEG_AS_SYNC_HANDLE handle, 
int **pp_andval, 
int **pp_orval,
int len0, 
int len1
)
{
	int idx_frame0, idx_frame1;
	for (idx_frame0 = 0; idx_frame0 < len0-(MPEG_AS_BLOCK_SIZE - 1); idx_frame0++) {
		for (idx_frame1 = 0; idx_frame1 < len1-(MPEG_AS_BLOCK_SIZE - 1); idx_frame1++) {
			int idx_frame2;
			int andval, orval;
			float score;

			if (handle->search_width > 0) {
				if (idx_frame1 < idx_frame0 - (int)(handle->search_width/handle->framestep_new)) {
					continue;
				}

				if (idx_frame1 > idx_frame0 + (int)(handle->search_width/handle->framestep_new)) {
					break;
				}
			}

			andval = 0;
			orval = 0;

			for (idx_frame2 = 0; idx_frame2 < MPEG_AS_BLOCK_SIZE; idx_frame2++) {
				andval += pp_andval[idx_frame0 + idx_frame2][idx_frame1 + idx_frame2];
				orval += pp_orval[idx_frame0 + idx_frame2][idx_frame1 + idx_frame2];
			}

			if (orval != 0) {
				score = (float)andval/(float)orval;
			} else {
				score = 0;
			}


			handle->p_scoresum[idx_frame1 - idx_frame0 + len0 - (MPEG_AS_BLOCK_SIZE - 1) - 1] += score;
			handle->p_num_point[idx_frame1 - idx_frame0 + len0 - (MPEG_AS_BLOCK_SIZE - 1) - 1]++;
		}
	}
	return;
}

static int check_route_length(
MPEG_AS_SYNC_HANDLE handle,
int	len0,
int	len1,
int	*p_max_num_point
) {
	int idx_lag;
	float maxlen_sec;
	int max_num_point;

	max_num_point = handle->p_num_point[0];

	for (idx_lag = 0; idx_lag < len0 -(MPEG_AS_BLOCK_SIZE - 1) + len1 -(MPEG_AS_BLOCK_SIZE - 1) - 1; idx_lag++) {
		if (handle->p_num_point[idx_lag] > max_num_point) {
			max_num_point = handle->p_num_point[idx_lag];
		}
	}

	maxlen_sec = (max_num_point + MPEG_AS_BLOCK_SIZE - 1) * handle->framestep_new/MPEG_AS_SAMPLING_RATE;

	if (maxlen_sec < MPEG_AS_MIN_LENGTH_OF_VALID_ROUTE || len0 < MPEG_AS_BLOCK_SIZE || len1 < MPEG_AS_BLOCK_SIZE) {
		fprintf(stderr, "input file size is too small (must be longer than %f sec)\n", (float)MPEG_AS_MIN_LENGTH_OF_VALID_ROUTE);
		return MPEG_AS_E_SHORT_INPUT;
	}

	*p_max_num_point = max_num_point;

	return	MPEG_AS_S_OK;
};

static void normalize_scoresum(
MPEG_AS_SYNC_HANDLE handle, 
int len0,
int len1,
int max_num_point
) 
{
	int idx_lag;
	for (idx_lag = 0; idx_lag < len0 -(MPEG_AS_BLOCK_SIZE - 1) + len1 -(MPEG_AS_BLOCK_SIZE - 1) - 1; idx_lag++) {
		if (handle->p_num_point[idx_lag] >= MPEG_AS_SCORESUM_THRESH) {
			if (handle->use_overlap_weighting == 0) {
				handle->p_scoresum[idx_lag] = handle->p_scoresum[idx_lag]/handle->p_num_point[idx_lag];
			} else {
				int block_offset = (MPEG_AS_BLOCK_SIZE - 1);
				float min_coef = (float)0.8;
				float max_coef = (float)1.0;
				float min_coef_pos = (float)0.3;
				float max_coef_pos = (float)0.5;
				if ((handle->p_num_point[idx_lag] + block_offset)> (max_num_point + block_offset)*max_coef_pos) {
					handle->p_scoresum[idx_lag] = handle->p_scoresum[idx_lag]/handle->p_num_point[idx_lag];
				} else if ((handle->p_num_point[idx_lag] + block_offset) > (max_num_point + block_offset)*min_coef_pos) {
					float grad = (max_coef - min_coef)/(max_coef_pos - min_coef_pos);
					float delta_pos = (float)(handle->p_num_point[idx_lag] + block_offset)/(float)(max_num_point + block_offset) - min_coef_pos;
					float coef = min_coef + grad * delta_pos;
					handle->p_scoresum[idx_lag] = coef*handle->p_scoresum[idx_lag]/handle->p_num_point[idx_lag];
				} else {
					handle->p_scoresum[idx_lag] = 0.0;
				}
			}
		} else {
			handle->p_scoresum[idx_lag] = 0.0;
		}
	}
	return;
}

static void get_max_val_of_scoresum(
MPEG_AS_SYNC_HANDLE handle, 
int len0, 
int len1, 
float *p_maxval, 
int *p_maxval_id
) 
{
	int		idx_lag;
	float	maxval = -1;
	int		maxval_id = 0;

	for (idx_lag = 0; idx_lag < len0 - (MPEG_AS_BLOCK_SIZE - 1) + len1 - (MPEG_AS_BLOCK_SIZE - 1) - 1; idx_lag++) {

		if ( handle->p_scoresum[idx_lag] != 0.0) {

			float score = handle->p_scoresum[idx_lag];
			if (score > maxval) {
				maxval_id = idx_lag;
				maxval = score;
			}
		}
	}

	*p_maxval = maxval;
	*p_maxval_id = maxval_id;

	return;
}


/* pick up candidates  */
void audio_sync_pick_up_results(
MPEG_AS_SYNC_HANDLE handle, 
float *p_score, 
int *p_delay_idx, 
float maxval, 
int maxval_id, 
int num_cand, 
int len0, 
int len1,
int *p_delay
) {
	int idx_cand;
	int	idx_lag;
	p_score[0] = maxval;
	p_delay_idx[0] = maxval_id;

	for (idx_cand = 1; idx_cand < num_cand; idx_cand++) {
		float maxval_tmp = -1;
		int maxval_id_tmp = 0;
		for (idx_lag = 0; idx_lag < len0 -(MPEG_AS_BLOCK_SIZE - 1) + len1 -(MPEG_AS_BLOCK_SIZE - 1) - 1; idx_lag++) {
			if (handle->p_scoresum[idx_lag] != 0.0) {
				float score = handle->p_scoresum[idx_lag];
				if (handle->p_scoresum[idx_lag] >= handle->p_scoresum[idx_lag-1] &&
					handle->p_scoresum[idx_lag] > handle->p_scoresum[idx_lag+1] &&
					score > maxval_tmp) {
						int is_valid = 1;
						int idx_tmp;
						for (idx_tmp = 0; idx_tmp < idx_cand; idx_tmp++) {
							if (!(idx_lag < p_delay_idx[idx_tmp] - MPEG_AS_SEARCH_AVOIDANCE_WIDTH || idx_lag > p_delay_idx[idx_tmp] + MPEG_AS_SEARCH_AVOIDANCE_WIDTH)) {
								is_valid = 0;
							}
						}

						if (is_valid) {
							maxval_id_tmp = idx_lag;
							maxval_tmp = score;
						}
				}
			}
		}

		p_delay_idx[idx_cand] = maxval_id_tmp;
		p_score[idx_cand] = maxval_tmp;
	}

	for (idx_cand = 0; idx_cand < num_cand; idx_cand++) {
		if (p_score[idx_cand] < 0) {
			p_delay[idx_cand] = 0;
		} else {
			p_delay[idx_cand] = -(p_delay_idx[idx_cand] - (len0 -(MPEG_AS_BLOCK_SIZE - 1) - 1))*((int)(handle->framestep_new)); 
		}
	}

	return;
}


/*****************************************************************************
	End of file
*****************************************************************************/
